; 1 CHANNEL PWM DECODER FOR R/C PULSE PROPORTIONAL RECEIVERS
;WITH 1 TO 2 MILLISECOND POSITIVE PULSE OUTPUT. 
; 
;WILLIAM MANGANARO
;07/05/96
;Modified: Waldi, 09/14/2000
;Zwei-Kanal-Version
;
;               
                             LIST P=16F84,F=INHX8M
#include <P16F84.INC>

;Checksum: 32BC
;----- USER DEFINED REGISTERS ---------------------------------------------


PCOUNTER	EQU		0CH 
FLAGS		EQU		0DH
V_RANGE		EQU		0EH
TEMP		EQU		0FH
CH_SEL_IN	EQU		10H 
COUNT1		EQU		11H
COUNT2		EQU		12H
CH_SEL_OUT	EQU		13H

;----- USER DEFINED BITS ---------------------------------------------------

CH1           	EQU 		2
CH2		EQU		3
VALID		EQU		0
OUT1		EQU		0
OUT2		EQU		1
LED1		EQU		4
LED2		EQU		5

;*******************************************
; MAIN CODE SEGMENT ************************
;*******************************************
;
;
	ORG     00H
;
	GOTO 	MAIN
;
	ORG	04H
;
TMROF	NOP	; TIMER0 OVERFLOW CONDITION DETECTED
        RETFIE
;
	ORG	20H
;
MAIN	BSF	STATUS,RP0 ; SELECT RAM PAGE 1
	MOVLW	0X33
   	MOVWF	OPTION_REG ; PRESCALE=TMRO, /16 MODE, TIMER OFF.
        MOVLW   0X1F
	MOVWF   TRISA ; PORTA INPUTS
        MOVLW   B'00001100'
        MOVWF	TRISB ; PORTB:  mixed
     	BCF     STATUS,RP0 ; SELECT RAM PAGE 0
        CLRF	PORTB ; TURN OFF OUTPUTS       
	MOVLW	0X01
	MOVWF	CH_SEL_OUT ; INITIALIZE CHANNEL SELECT TO CHANNEL 1
	MOVLW	0X04
	MOVWF	CH_SEL_IN ; INITIALIZE CHANNEL SELECT TO CHANNEL 1
SYNC	MOVLW	0X0F
 	MOVWF   PCOUNTER ; SET UP VALID PULSE COUNTER FOR 15
MVL     BSF	STATUS,RP0 ; PAGE 1 SELECT
	BSF	OPTION_REG,T0CS ; TURN TIMER OFF
	BCF	STATUS,RP0 ; PAGE 0 SELECT
;
;
;
;
; WAIT FOR INPUT TO GO LOGIC HIGH
;
J2	MOVF	PORTB,W
	ANDLW	0X0C
	ANDWF	CH_SEL_IN,W
	BTFSC	STATUS,Z
	GOTO	J2
;
;
	CLRF	TMR0 ; CLEAR TIMER
	BCF     INTCON,T0IF ; CLEAR TIMER OVERFLOW
	BSF	STATUS,RP0 ; PAGE 1 SELECT
	BCF	OPTION_REG,T0CS ; TURN TIMER ON
	BCF	STATUS,RP0 ; PAGE 0 SELECT
J3	BTFSS	INTCON,T0IF ; IS TMR0 OVERFLOW SET?
	GOTO    J4
        CALL	SELOFF ; TURN SELECTED CHANNEL OFF
        CALL	CSWITCH ; SWITCH TO OTHER CHANNEL
	GOTO    SYNC ; GO BACK TO SYNC
J4	MOVF	PORTB,W ; READ PORT A
	ANDLW	0X0C ; MASK BITS 2 THRU 7
	ANDWF	CH_SEL_IN,W ; AND WITH CHANNEL SELECT MASK
	BTFSS	STATUS,Z ; HAS SELECTED CHANNEL GONE LOW?
	GOTO	J3 ; RETEST
	BSF	STATUS,RP0 ; PAGE 1 SELECT
	BSF	OPTION_REG,T0CS ; TURN TIMER OFF
	BCF	STATUS,RP0 ; PAGE 0 SELECT      	
	NOP
	NOP
	MOVF	TMR0,W
	MOVWF	TEMP ; COPY TMR0 INTO TEMP REGISTER
	CALL	PW_CHK ; CHECK PULSE WIDTH
	BTFSC	FLAGS,VALID ; VALID=1=VALID PULSE WIDTH
	GOTO	J5
        CALL	SELOFF ; TURN SELECTED CHANNEL OFF
	CALL	CSWITCH ; LOOK AT OTHER CHANNEL
	CALL	DELAY ; WAIT 6 MILLISECONDS BEFORE SAMPLING CHANNEL
	GOTO    SYNC ; START OVER
J5      DECFSZ	PCOUNTER,F ; HAS THE PULSE WIDTH BEEN VALID FOR 15 PULSES?
	GOTO    MVL ; GO TO THE MAIN VALIDATION LOOP
	CALL	SELON ; TURN SELECTED CHANNEL ON
	CALL	CSWITCH ; LOOK AT OTHER CHANNEL
	CALL	DELAY ; WAIT 6 MILLISECONDS BEFORE SAMPLING CHANNEL
	GOTO    SYNC
;

        


;*******************************************
; SUBROUTINES  *****************************
;*******************************************
;
;
; ******************************************************
; * PULSE WIDTH CHECK: CHECK TO SEE IF TIMER0 REGISTER *
; * HOLDS A VALID TIME PERIOD MEASUREMENT BETWEEN 1.79 *
; * AND 2.5 MILLISECOMNDS. IF SO, SET A VALID PW FLAG  *
; * FOR USE BY MAIN VALIDATION LOOP                    *
; ******************************************************
;
;
PW_CHK	BCF	FLAGS,VALID ; CLEAR THE PULSE WIDTH VALID FLAG
	MOVLW	0X70 ; SET TRIP POINT (70H = 112 DEC.) 1.79 MILLI. SEC.
	MOVWF	V_RANGE
 
VL	MOVF	V_RANGE,W
	XORWF	TMR0,F ; IS PW WITHIN VALID RANGE
	BTFSC   STATUS,Z
	BSF	FLAGS,VALID ; IF VALID, SET VALID FLAG
	XORLW	0X9C ; IS VALID RANGE COUNTER=MAXRANGE (9CH = 2.5 MILLISEC)
	BTFSC	STATUS,Z
	RETURN ; RETURN IF AT END OF RANGE
	INCF	V_RANGE,F ; INCREMENT VALID RANGE COUNTER
	MOVF	TEMP,W ; TEMP HOLDS TMR0 VALUE
	MOVWF	TMR0
	GOTO	VL
;
;
; CHANNEL SELECTION ROUTINE
;
;
CSWITCH	
	COMF	CH_SEL_OUT,F ; COMPLIMENT THE CHANNEL SELECT REGISTER
	MOVLW	0X03
	ANDWF	CH_SEL_OUT,F ; MASK BITS 0 & 1
	COMF	CH_SEL_IN,F ; COMPLIMENT THE CHANNEL SELECT REGISTER
	MOVLW	0X0C
	ANDWF	CH_SEL_IN,F ; MASK BITS 2 & 3
	RETURN
;
;
; TURN SELECTED CHANNEL ON ROUTINE
;
;
SELON	
	BTFSS	CH_SEL_OUT,0 ; IS CHANNEL 1 SELECTED?
	GOTO	J6
	BSF	PORTB,OUT1 ; OUTPUT 1 ON IF SELECTED
;	BSF	PORTB,LED1 ; SET CHANNEL 1 ACTIVE LED ON
        RETURN
J6	BSF	PORTB,OUT2 ; OUTPUT 2 ON IF SELECTED
;        BSF	PORTB,LED2 ; SET CHANNEL 2 ACTIVE LED ON        
	RETURN
;
;
; TURN SELECTED CHANNEL OFF ROUTINE
;
;
SELOFF	
	BTFSS	CH_SEL_OUT,0 ; IS CHANNEL 1 SELECTED?
	GOTO	J7
	BCF	PORTB,OUT1 ; OUTPUT 1 OFF IF SELECTED
;	BCF	PORTB,LED1 ; SET CHANNEL 1 ACTIVE LED OFF	
	RETURN
J7	BCF	PORTB,OUT2 ;OUTPUT 2 OFF IF SELECTED
;	BCF	PORTB,LED2 ; SET CHANNEL 1 ACTIVE LED ON
	RETURN
;
;
DELAY	MOVLW	0X00
	MOVWF	COUNT1
	MOVLW	0X06
	MOVWF	COUNT2
DLY1	INCF	COUNT1,F
	BTFSS	STATUS,Z
	GOTO	DLY1
	DECFSZ	COUNT2,F
	GOTO	DLY1
	RETURN	
;
	END

